use redis::Commands;
use tracing::debug;

use crate::{error::NtitledError, File};

const DEFAULT_EXPIRE: u64 = 60 * 60 * 24 * 3;
pub trait Cache {
    fn set_file_meta(&mut self, file: &File) -> Result<(), NtitledError>;

    fn get_file_meta(&mut self, file_name: &str) -> Result<Option<File>, NtitledError>;

    fn del_file_meta(&mut self, file_name: &str) -> Result<(), NtitledError>;
}

impl Cache for redis::Client {
    fn set_file_meta(&mut self, file: &File) -> Result<(), NtitledError> {
        debug!("Caching: {}", file.name);
        let key = &file.name;
        let val = serde_json::to_string(file)?;
        self.set_ex(key, val, DEFAULT_EXPIRE)?;
        Ok(())
    }

    fn get_file_meta(&mut self, file_name: &str) -> Result<Option<File>, NtitledError> {
        let val: Option<String> = self.get(file_name)?;
        let Some(val) = val.as_ref() else {
            return Ok(None);
        };
        let val = serde_json::from_str(val)?;
        Ok(Some(val))
    }

    fn del_file_meta(&mut self, file_name: &str) -> Result<(), NtitledError> {
        debug!("Clearing: {}", file_name);
        self.del(file_name)?;
        Ok(())
    }
}
