use std::{num::ParseIntError, string::FromUtf8Error};

use axum::{http::StatusCode, response::IntoResponse};
use redis::RedisError;
use thiserror::Error;
use tracing::error;

#[derive(Debug, Error)]
pub enum NtitledError {
    #[error("{0}")]
    IO(#[from] std::io::Error),

    #[error("{0}")]
    Serde(#[from] serde_json::Error),

    #[error("{0}")]
    Reqwest(#[from] reqwest::Error),

    #[error("{0}")]
    Message(String),

    #[error("{0}")]
    Utf8(#[from] FromUtf8Error),

    #[error("{0}")]
    Parse(#[from] ParseIntError),

    #[error("{0}")]
    Redis(#[from] RedisError),
}

impl IntoResponse for NtitledError {
    fn into_response(self) -> axum::response::Response {
        error!("Error: {self}");
        match self {
            NtitledError::IO(e) => {
                (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response()
            }
            NtitledError::Serde(e) => {
                (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response()
            }
            NtitledError::Reqwest(e) => {
                (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response()
            }
            NtitledError::Parse(e) => {
                (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response()
            }
            NtitledError::Redis(e) => {
                (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response()
            }
            NtitledError::Utf8(e) => (StatusCode::BAD_REQUEST, e.to_string()).into_response(),
            NtitledError::Message(m) => (StatusCode::BAD_REQUEST, m).into_response(),
        }
    }
}
