//! https://trac.opensubtitles.org/projects/opensubtitles/wiki/HashSourceCodes#RUST

use std::{
    fs::File,
    io::{self, BufReader, Read, Seek, SeekFrom},
};

const HASH_BLK_SIZE: u64 = 65536;

pub fn compute_hash(fname: &str) -> std::io::Result<String> {
    let fsize = std::fs::metadata(fname).unwrap().len();
    if fsize > HASH_BLK_SIZE {
        let file = File::open(fname).unwrap();
        create_hash(file, fsize)
    } else {
        println!("Error");
        Err(io::Error::new(
            io::ErrorKind::InvalidInput,
            "File is smaller than block size, cannot determine hash",
        ))
    }
}

fn create_hash(file: File, fsize: u64) -> Result<String, std::io::Error> {
    let mut buf = [0u8; 8];
    let mut word: u64;

    let mut hash_val: u64 = fsize; // seed hash with file size

    let iterations = HASH_BLK_SIZE / 8;

    let mut reader = BufReader::with_capacity(HASH_BLK_SIZE as usize, file);

    for _ in 0..iterations {
        reader.read_exact(&mut buf)?;
        unsafe {
            word = std::mem::transmute(buf);
        };
        hash_val = hash_val.wrapping_add(word);
    }

    reader.seek(SeekFrom::Start(fsize - HASH_BLK_SIZE))?;

    for _ in 0..iterations {
        reader.read_exact(&mut buf)?;
        unsafe {
            word = std::mem::transmute(buf);
        };
        hash_val = hash_val.wrapping_add(word);
    }

    let hash_string = format!("{:01$x}", hash_val, 16);

    Ok(hash_string)
}
